package net.pricefx.formulaengine;

import java.util.Collection;
import java.util.Map;

import com.googlecode.genericdao.search.Filter;

import net.pricefx.domain.CompensationAgreementPayoutRecord;
import net.pricefx.domain.PayoutRecord;
import net.pricefx.service.PersistenceServiceException;

/**
 * API that can be accessed via the 'payoutRecords' binding, only available in a RebateRecord calculation context
 * (i.e. when calculating a RebateReocrd in a RebateRecordCalculation task). * 
 * <p>
 * Example groovy:
 * <blockquote>
 * <pre>
 * def rr = rebateRecords.get("Q1")		// api.currentItem()
 * for (def i=0; i &lt; 100000; i++) {
 * 		payoutRecords.add("Accrual", "Q1-"+i, [attribute1: rr?.attribute1, attribute2: rr?.attribute2, attribute3: rr?.attribute3] )
 * 		payoutRecords.add("Summary", "Q1-"+i, [attribute1: rr?.attribute1, attribute2: rr?.attribute2, attribute3: rr?.attribute3] )
 * } 
 * </pre>
 * </blockquote>
 * Payment records managed through this API are always associated with the current RebateRecord.
 */
public interface PayoutRecordManager {

	/**
	 * Retreives the payout record, if any, with the given type and name, linked to the current RebateRecord.
	 * @param type
	 * @param name
	 * @return	The payment record converted to its map representation
	 */
	CompensationAgreementPayoutRecord get(String type, String name);

	/**
	 * Add a previously non-existing PayoutRecord, identified by type and name
	 * @param type	A free-text type for the record.  While it can be freely set, the usual patterns is that there are few different payment record types.
	 * @param name  A free-text name for the record, unique withing the (RebateRecord,type) namespace.
	 */
	void add(String type, String name, final Map<?,?>... attributeValues) throws PersistenceServiceException;
	
	/**
	 * Updates the existing PayoutRecord, if any, identified by the given type and name
	 * @param type
	 * @param name
	 * @param attributeValues	Attribute to value map (ex. {attribute1: "ABC", ...}).  If an attribute is to be set to null, it needs to be included in the map as such.
	 * @throws PersistenceServiceException	When a PayoutRecord.attribute cannot be set to the given value
	 */
	void update(String type, String name, final Map<?,?>... attributeValues) throws PersistenceServiceException;
	
	/**
	 * Convenience method, adding a PayoutRecord if one with the given type and name does not yest exist, and updating it otherwise.
	 * Important note:  in bulk-load mode, there is no advantage to be had from calling addOrUpdate because the bulk-load always
	 * performs an insert (for a new record) or replace (for an existing record), regardless of weather the record was added with add() or addOrUpdate().
	 * @param type
	 * @param name
	 * @param attributeValues	Attribute to value map (ex. {attribute1: "ABC", ...}).  If an attribute is to be set to null, it needs to be included in the map as such.
	 * @throws PersistenceServiceException	When a PayoutRecord.attribute cannot be set to the given value
	 */
	void addOrUpdate(String type, String name, final Map<?,?>... attributeValues) throws PersistenceServiceException;
	
	/**
	 * Retreive PayoutRecords linked to the current RebateRecord, adhering to one or more optional Filter criteria
	 * @param filters
	 * @return	Collection of PayoutRecords matching the above conditions, each in its map representation.
	 */
	Collection<CompensationAgreementPayoutRecord> find(Filter... filters);
	
	/**
	 * Delete PayoutRecords linked to the current RebateRecord, adhering to one or more optional Filter criteria
	 * @param filters
	 * @return	The number of actually deleted records
	 */
	int delete(Filter... filters);

	/**
	 * Switches to bulk-load mode, which is recommended when the volume of records to be added or updated exceeds 100k
	 * (in regular mode, 100k records are created in around 10min).
	 * Important note:  When existing records are bulk-loaded, the original records are replaced in full, which is different
	 * to the regular node, where only the passed in attributes/value pairs are updated.
	 */
	void bulkLoadMode();

	/**
	 * To be called when all records have been created and/or updated.
	 * If skipped the more recent additions and updates will not be persisted in the database.
	 */
	void flush();

}
