package net.pricefx.formulaengine.optimization;

import com.googlecode.genericdao.search.Filter;

public interface ModelFilterTreesContextReadOnly {
    /**
     * @param name The name of a {@link FilterTree} owned by this model
     * @return A {@link FilterTree} or {@code null} if it doesn't exist
     */
    FilterTree tree(String name);

    /**
     * Represents a tree that can be fetched in various manners.
     */
    interface FilterTree {
        /**
         * @return the name of this tree
         */
        String name();

        /**
         * Fetch the whole tree.
         * <p>
         * Can be walked down via the children and up via the parent.
         *
         * @return the root node
         */
        FilterTreeNode fetch();

        /**
         * Fetch one branch of the tree, starting from the given node up to the root.
         * <p>
         * Can be walked up via the parent of the nodes and down via the (unique) child of the nodes.
         *
         * <p>For example, given the following tree:
         * <ul>
         *     <li>root (id=1)
         *     <ul>
         *         <li>a (id=2)
         *         <ul>
         *             <li>c (id=4)</li>
         *             <li>d (id=5)</li>
         *             </ul></li>
         *         <li>b (id=3)
         *         <ul>
         *             <li>e (id=6)</li>
         *             <li>f (id=7)</li>
         *             </ul></li>
         *     </ul>
         *     </li>
         * </ul>
         * </p>
         *
         * <p>{@code tree.fetchBranch(5)} will fetch:
         * <ul>
         *     <li>root (id=1)
         *     <ul>
         *         <li>a (id=2)
         *         <ul>
         *             <li>d (id=5)</li>
         *             </ul></li>
         *     </ul>
         *     </li>
         * </ul>
         * and return node {@code d}</p>
         *
         * <p>{@code tree.fetchBranch(3)} will fetch:
         * <ul>
         *     <li>root (id=1)
         *     <ul>
         *         <li>b (id=3)</li>
         *     </ul>
         *     </li>
         * </ul>
         * and return node {@code b}</p>
         *
         * @param leafId the id of the starting node for the branch
         * @return the leaf of the fetched branch
         */
        FilterTreeNode fetchBranch(long leafId);
    }

    /**
     * Represents an in-memory tree that can be walked up and down.
     */
    interface FilterTreeNode {
        /**
         * @return the node id
         */
        long id();

        /**
         * @return the parent node or {@code null} if this node is the tree root
         */
        FilterTreeNode parent();

        /**
         * @return the node name
         */
        String name();

        /**
         * @return the node label
         */
        String label();

        /**
         * @return the node filter
         */
        Filter filter();

        /**
         * @return the children of the node
         */
        Iterable<FilterTreeNode> children();
    }
}
